//=========================================================================================================================
//Monoscopic Augmented Reality demo by Maxim Lysak, Viktor Kuropyatnik (VRM team http://3dvrm.com/)
//Based on FLAR Toolkit (alchemy branch) by Saqoosha, Ryo Iizuka and Papervision3D by Carlos Ulloa Matesanz
//
//Get latest FLAR Toolkit alchemy branch here (use SVN):
//http://www.libspark.org/svn/as3/FLARToolKit/branches/alchemy
//
//Get latest Papervision3D here (use SVN):
//http://papervision3d.googlecode.com/svn/trunk
//
//Add ".../src" folder from your Flar Alchemy directory,
//and  ".../as3/trunk/src" from Papervision3D directory
//to your ActionScript source path.
//Do this by: Project properties -> ActionScript Build Path -> Source path -> Add Folder...
//
//You also must include FLARToolKit.swc to your project to compile
//Do this by: Project properties -> ActionScript Build Path -> Library path -> Add SWC...
//=========================================================================================================================

package
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Scene;
	import flash.display.Sprite;
	import flash.display.StageDisplayState;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.media.Camera;
	import flash.media.Video;
	import flash.utils.ByteArray;
	
	import org.libspark.flartoolkit.core.FLARCode;
	import org.libspark.flartoolkit.core.FLARParam;
	import org.libspark.flartoolkit.core.FLARRgbRaster;
	import org.libspark.flartoolkit.core.FLARTransMatResult;
	import org.libspark.flartoolkit.detector.FLARSingleMarkerDetector;
	import org.libspark.flartoolkit.support.pv3d.FLARCamera3D;
	import org.libspark.flartoolkit.support.pv3d.FLARMarkerNode;
	import org.papervision3d.core.math.Matrix3D;
	import org.papervision3d.objects.parsers.DAE;
	import org.papervision3d.render.BasicRenderEngine;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.view.Viewport3D;
	
	[SWF(width="640", height="480", backgroundColor="#000000")]
	
	public class FlarAlchemyMonoscopic extends Sprite
	{
		[Embed(source="ARMarker16x16.pat", mimeType="application/octet-stream")] //Marker pattern
		private var pattern:Class;
		
		[Embed(source="camera_para.dat", mimeType="application/octet-stream")]   //Camera configuration file
		private var params:Class;		
		
		private var fparams:FLARParam;
		private var mpattern:FLARCode;
		private var vid1:Video;
		private var cam1:Camera;
		private var bmd1:BitmapData;
		private var capture1:Bitmap;
		
		private var raster1:FLARRgbRaster;		
		private var detector1:FLARSingleMarkerDetector;		
		private var scene:Scene3D;
		private var camera:FLARCamera3D;		
		private var container:FLARMarkerNode;
		private var vp1:Viewport3D;
		private var bre:BasicRenderEngine;		
		private var trans1:FLARTransMatResult;

		protected var canvasWidth:int;
		protected var canvasHeight:int;
		protected var captureWidth:int;
		protected var captureHeight:int;	
		protected var videoWidth:int;
		protected var videoHeight:int;
		
		private var skipframe:int;		
		private var _threshold:int = 110; //Change threshold to	make merker "visible" to camera (110 default)			
		private var collada_model:DAE;

		public function FlarAlchemyMonoscopic()
		{
			//Resolution of video for marker recognition
			this.captureWidth = 320;
			this.captureHeight = 240;
			
			//Resolution for video output			
			this.videoWidth = 640;
			this.videoHeight = 480;			
			
			//Resolution of canvas
			this.canvasWidth = 640
			this.canvasHeight = 480;			
			
			setupFLAR();
			setupCamera();
			setupBitmap();			
			setupDAE();			
			setupPV3D();	
			
			addEventListener(Event.ENTER_FRAME, loop);
			stage.addEventListener(MouseEvent.CLICK, _handleClick); //On click - go fullscreen
		}
		
		private function goFullScreen():void
		{
		    if (stage.displayState == StageDisplayState.NORMAL) {
				        stage.displayState=StageDisplayState.FULL_SCREEN;
				    } else {
				        stage.displayState=StageDisplayState.NORMAL;
				    }
		}		
		
		private function _handleClick(e:MouseEvent):void
		{
		    goFullScreen();
		}		
		
		private function setupDAE():void
		{
			//Load collada model
			collada_model = new DAE();
			collada_model.load("model/cow.dae");
			//And setup it's initial parameters
			collada_model.scale = 40;			
			collada_model.rotationX = 90;		
		}
		
		private function setupFLAR():void
		{
			//Initialize FLAR Toolkit by loading a camera parameters and AR marker
			fparams = new FLARParam();
			fparams.loadARParamFile(new params() as ByteArray);
			mpattern = new FLARCode(16, 16);
			mpattern.loadARPattFromFile(new pattern());
		}
		
		private function setupCamera():void
		{
			//Initialize two video streams for stereoscopic vision
			vid1   = new Video(captureWidth,captureHeight);
		
			//Init camera
			cam1 = Camera.getCamera();
			cam1.setMode(captureWidth,captureHeight,30,false);
			vid1.attachCamera(cam1);
			//Re-init resolution here, if you want to visualize background video feed with higher rez
			cam1.setMode(videoWidth,videoHeight,30,false);
			cam1.setQuality(0,0);
			
			//Set video-output viewport
			vid1.width = canvasWidth;
			vid1.height = canvasHeight;		

			addChild(vid1);
		}
		
		private function setupBitmap():void
		{
			// Setup ARToolkit			
			bmd1 = new BitmapData(captureWidth, captureHeight,false,0);
			bmd1.draw(vid1);
			raster1 = new FLARRgbRaster(captureWidth,captureHeight);
			raster1.setBitmapData(bmd1);
			
			fparams.changeScreenSize(captureWidth, captureHeight);
			// Setup marker detector
			// Note - 80 is real-world size of the marker size in millimiters
			// This value is crucial for correct stereoscopic visualization
			detector1 = new FLARSingleMarkerDetector(fparams, mpattern, 80);	
			
			// Set continue mode to true corrects spontaneous jumps of virtual objects
			detector1.setContinueMode(true);			
		}		
		
		private function setupPV3D():void
		{
			//Setup Papervision3D
			scene = new Scene3D();
			camera = new FLARCamera3D(fparams);
			container = new FLARMarkerNode(1);
			scene.addChild(container);
			
			//--------------------------------------------------Draw your scene here
			container.addChild(collada_model); //Our previously loaded collada model
			//----------------------------------------------------------------------
			
			//Initialize render engine
			bre = new BasicRenderEngine();
			//And transformation matrix
			trans1 = new FLARTransMatResult();
	
			//Setup 3D viewport
			vp1 = new Viewport3D(captureWidth,captureHeight);
			vp1.scaleX = this.canvasWidth / this.captureWidth;
			vp1.scaleY = this.canvasHeight / this.captureHeight;
			vp1.x = -4; // 4pix ???
			addChild(vp1);			
		}		
		
		private function loop(e:Event):void
		{		
			//Refresh video bitmap and raster for recognition
			bmd1.draw(vid1);			
			raster1.setBitmapData(bmd1);
			
			// Marker detection
			var detected:Boolean = false;
			
			try
			{	//Detect marker
				detected = (detector1.detectMarkerLite(raster1, _threshold) && detector1.getConfidence() > 0.3)	
			}
			catch(e:Error){}
			
			if (detected){
				container.visible = true;		
				//Get camera transformation object from detector
				detector1.getTransformMatrix(trans1);			
				//Set transformation camera matrix
				container.setTransformMatrix(trans1);
				//Render the scene in left viewport
				bre.renderScene(scene, camera, vp1);
				
				skipframe = 0;				
			}else{
				//If marker isn't detected, count 30 frames and stop viewing the scene
				skipframe = skipframe + 1;
				container.setTransformMatrix(trans1);				
				bre.renderScene(scene, camera, vp1);								
				
				if (skipframe>30){
					//This was made to prevent spontaneous flickering of the scene in bad lighting conditions
					container.visible = false;
					container.setTransformMatrix(trans1);					
					bre.renderScene(scene, camera, vp1);	
				}
			}					
			//-----------------------------------------------------------------------------------------------------------------------------		
			
		}
	}
}