//=========================================================================================================================
//Monoscopic Augmented Reality demo by Maxim Lysak, Viktor Kuropyatnik (VRM team http://3dvrm.com/)
//Based on FLAR Toolkit by Saqoosha, Ryo Iizuka and Papervision3D by Carlos Ulloa Matesanz
//
//Get latest FLAR Toolkit here (use SVN):
//http://www.libspark.org/svn/as3/FLARToolKit/trunk/
//
//Get latest Papervision3D here (use SVN):
//http://papervision3d.googlecode.com/svn/trunk
//
//Also ".../src" and ".../libs/NyARToolKitAS3/src" folders from your Flar Alchemy directory,
//and  ".../as3/trunk/src" from Papervision3D directory
//to your ActionScript source path.
//Do this by: Project properties -> ActionScript Build Path -> Source path -> Add Folder...
//=========================================================================================================================

package
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.PixelSnapping;
	import flash.display.Scene;
	import flash.display.Sprite;
	import flash.display.StageDisplayState;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.media.Camera;
	import flash.media.Video;
	import flash.utils.ByteArray;
	
	import org.libspark.flartoolkit.core.FLARCode;
	import org.libspark.flartoolkit.core.analyzer.raster.threshold.FLARRasterThresholdAnalyzer_SlidePTile;
	import org.libspark.flartoolkit.core.param.FLARParam;
	import org.libspark.flartoolkit.core.raster.rgb.FLARRgbRaster_BitmapData;
	import org.libspark.flartoolkit.core.transmat.FLARTransMatResult;
	import org.libspark.flartoolkit.detector.FLARSingleMarkerDetector;
	import org.libspark.flartoolkit.support.pv3d.FLARBaseNode;
	import org.libspark.flartoolkit.support.pv3d.FLARCamera3D;
	import org.papervision3d.objects.parsers.DAE;
	import org.papervision3d.render.BasicRenderEngine;
	import org.papervision3d.render.LazyRenderEngine;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.view.Viewport3D;
	
	
	[SWF(width="640", height="480", frameRate="60", backgroundColor="#000000")]
	
	public class FlarMonoscopic extends Sprite
	{
		[Embed(source="ARMarker8x8.pat", mimeType="application/octet-stream")] //Marker pattern
		private var pattern:Class;
		
		[Embed(source="camera_para.dat", mimeType="application/octet-stream")] //Camera configuration file
		private var params:Class;		
		
		private var fparams:FLARParam;
		private var mpattern:FLARCode;
		private var vid:Video;
		private var cam:Camera;
		private var bmd:BitmapData;
		private var capture:Bitmap;
		
		private var raster:FLARRgbRaster_BitmapData;
		private var detector:FLARSingleMarkerDetector;
		private var scene:Scene3D;
		private var camera:FLARCamera3D;
		private var container:FLARBaseNode;
		private var vp:Viewport3D;
		private var bre:BasicRenderEngine;	
		private var trans:FLARTransMatResult;
		
		protected var canvasWidth:int;
		protected var canvasHeight:int;
		protected var captureWidth:int;
		protected var captureHeight:int;	
		protected var videoWidth:int;
		protected var videoHeight:int;
		
		private var skipframe:int;
		private var _threshold:int = 110; //Change threshold to	make merker "visible" to camera (110 default)				
		private var collada_model:DAE;
		
		public function FlarMonoscopic()
		{
			//Resolution of video for marker recognition			
			this.captureWidth = 160;
			this.captureHeight = 120;
			
			//Resolution for video output				
			this.videoWidth = 640;
			this.videoHeight = 480;			
			
			//Resolution of canvas			
			this.canvasWidth = 640
			this.canvasHeight = 480;			
			
			setupFLAR();
			setupCamera();
			setupBitmap();
			
			setupDAE();			
			setupPV3D();
				
			addEventListener(Event.ENTER_FRAME, loop);
			stage.addEventListener(MouseEvent.CLICK, _handleClick); //On click - go fullscreen			
		}
		
		private function goFullScreen():void
		{
			if (stage.displayState == StageDisplayState.NORMAL) {
				stage.displayState=StageDisplayState.FULL_SCREEN;
			} else {
				stage.displayState=StageDisplayState.NORMAL;
			}
		}		
		
		private function _handleClick(e:MouseEvent):void
		{
			goFullScreen();
		}			
		
		private function setupDAE():void
		{
			//Load collada model			
			collada_model = new DAE();
			collada_model.load("model/cow.dae");
			//And setup it's initial parameters		
			collada_model.scale = 40;
			collada_model.rotationX = 90;		
		}
		
		private function setupFLAR():void
		{
			//Initialize FLAR Toolkit by loading a camera parameters and AR marker					
			fparams = new FLARParam();
			fparams.loadARParam(new params() as ByteArray);
			mpattern = new FLARCode(8, 8);
			mpattern.loadARPatt(new pattern());
		}
		
		private function setupCamera():void
		{
			//Initialize video stream				
			vid = new Video(captureWidth,captureHeight);
			//Init camera			
			cam = Camera.getCamera();
			cam.setMode(captureWidth,captureHeight,99,false);
			vid.attachCamera(cam);
			//Set video-output viewport	
			vid.width = canvasWidth;
			vid.height = canvasHeight;		
			//Re-init resolution here, if you want to visualize background video feed with higher rez
			cam.setMode(videoWidth,videoHeight,99,false);
			addChild(vid);		
		}
		
		private function setupBitmap():void
		{
			// setup ARToolkit
			bmd = new BitmapData(captureWidth, captureHeight);
			bmd.draw(vid);
			raster = new FLARRgbRaster_BitmapData(bmd);
			fparams.changeScreenSize(captureWidth, captureHeight);
			// Setup marker detector
			// Note - 80 is real-world size of the marker size in millimiters
			// This value is crucial for correct stereoscopic visualization					
			detector = new FLARSingleMarkerDetector(fparams, mpattern, 80);
			// Set continue mode to true corrects spontaneous jumps of virtual objects				
			detector.setContinueMode(true);					
		}		
		
		private function setupPV3D():void
		{
			//Setup Papervision3D				
			scene = new Scene3D();
			camera = new FLARCamera3D(fparams);
			container = new FLARBaseNode();
			scene.addChild(container);
			
			//--------------------------------------------------Draw your scene here
			container.addChild(collada_model); //Our previously loaded collada model
			//----------------------------------------------------------------------
			
			//Initialize render engine	
			bre = new BasicRenderEngine();
			//And transformation matrix
			trans = new FLARTransMatResult();
			
			//Setup 3D viewport
			vp = new Viewport3D(captureWidth,captureHeight);
			addChild(vp);
			vp.scaleX = this.canvasWidth / this.captureWidth;
			vp.scaleY = this.canvasHeight / this.captureHeight;
			vp.x = -4; // 4pix ???			
		}		
		
		private function loop(e:Event):void
		{
			//Refresh video bitmap and raster for recognition					
			bmd.draw(vid);
			// Marker detection	
			var detected:Boolean = false;
			
			try
			{	//Detect marker
				detected = (detector.detectMarkerLite(raster, _threshold) && detector.getConfidence() > 0.5)
			}
			catch(e:Error){}
			
			if (detected){
				container.visible = true;
				//Get camera transformation matrix object from detector						
				detector.getTransformMatrix(trans);				
				//Set transformation camera matrix for the left eye				
				container.setTransformMatrix(trans);
				//Render the scene in left viewport				
				bre.renderScene(scene, camera, vp);	
				skipframe = 0;					
			}else{
				//If marker isn't detected, count 30 frames and stop viewing the scene				
				skipframe = skipframe + 1;
				container.setTransformMatrix(trans);				
				bre.renderScene(scene, camera, vp);								
				
				if (skipframe>30){
					//This was made to prevent spontaneous flickering of the scene in bad lighting conditions					
					container.visible = false;
					container.setTransformMatrix(trans);					
					bre.renderScene(scene, camera, vp);
				}
			}						
			
		}
	}
}
